/*
 * snes_to_usb.c
 *
 * Created: 7/22/2012 2:15:53 PM
 *  Author: don
 */ 

#include "snes-to-usb.h"
#include "ctrlr.h"
#include "Keyboard.h"

// received button press mask from controller
uint16_t buttons;

int main(void)
{
	// setup LED outputs
	LED_PORT_DIR |= STATUS_LED | ACTION_LED;
	LED_PORT &= ~(STATUS_LED | ACTION_LED);		// both LEDs off until USB initialized
	ctrlr_init ();
	USB_Init ();
	sei ();
	
	// main loop
	ctrlr_start_read ();
	
	while (true) {
		KeyboardTasks ();
		if (ctrlr_read_poll (&buttons)) {
			if (buttons != 0)
				LED_PORT |= ACTION_LED;
			else
				LED_PORT &= ~ACTION_LED;
		
			ctrlr_start_read ();
		}		
	}
}

void EVENT_USB_Device_Connect(void)
{
	LED_PORT |= STATUS_LED;
}

void EVENT_USB_Device_Disconnect(void)
{
	LED_PORT &= ~STATUS_LED;
}

/** Event handler for the library USB Configuration Changed event. */
void EVENT_USB_Device_ConfigurationChanged(void)
{
	bool ConfigSuccess = true;

	ConfigSuccess &= HID_Device_ConfigureEndpoints(&Keyboard_HID_Interface);

	USB_Device_EnableSOFEvents();

	if (! ConfigSuccess) LED_PORT &= ~STATUS_LED;
}


/** HID class driver callback function for the creation of HID reports to the host.
 *
 *  \param[in]     HIDInterfaceInfo  Pointer to the HID class interface configuration structure being referenced
 *  \param[in,out] ReportID    Report ID requested by the host if non-zero, otherwise callback should set to the generated report ID
 *  \param[in]     ReportType  Type of the report to create, either HID_REPORT_ITEM_In or HID_REPORT_ITEM_Feature
 *  \param[out]    ReportData  Pointer to a buffer where the created report should be stored
 *  \param[out]    ReportSize  Number of bytes written in the report (or zero if no report is to be sent
 *
 *  \return Boolean true to force the sending of the report, false to let the library determine if it needs to be sent
 */
bool CALLBACK_HID_Device_CreateHIDReport(USB_ClassInfo_HID_Device_t* const HIDInterfaceInfo, uint8_t* const ReportID,
                                         const uint8_t ReportType, void* ReportData, uint16_t* const ReportSize)
{
	USB_KeyboardReport_Data_t* KeyboardReport = (USB_KeyboardReport_Data_t*)ReportData;

	uint8_t UsedKeyCodes = 0;

	if (buttons & BUTTON_A)
		KeyboardReport->KeyCode[UsedKeyCodes++] = HID_KEYBOARD_SC_X;

	if (buttons & BUTTON_B)
		KeyboardReport->KeyCode[UsedKeyCodes++] = HID_KEYBOARD_SC_Z;
		
	if (buttons & BUTTON_X)
		KeyboardReport->KeyCode[UsedKeyCodes++] = HID_KEYBOARD_SC_S;
		
	if (buttons & BUTTON_Y)
		KeyboardReport->KeyCode[UsedKeyCodes++] = HID_KEYBOARD_SC_A;
		
	if (buttons & BUTTON_SEL)
		KeyboardReport->Modifier = HID_KEYBOARD_MODIFER_RIGHTSHIFT;
		
	if (buttons & BUTTON_START)
		KeyboardReport->KeyCode[UsedKeyCodes++] = HID_KEYBOARD_SC_ENTER;
		
	if (buttons & BUTTON_U)
		KeyboardReport->KeyCode[UsedKeyCodes++] = HID_KEYBOARD_SC_UP_ARROW;
		
	if (buttons & BUTTON_D)
		KeyboardReport->KeyCode[UsedKeyCodes++] = HID_KEYBOARD_SC_DOWN_ARROW;

	if (buttons & BUTTON_L)
		KeyboardReport->KeyCode[UsedKeyCodes++] = HID_KEYBOARD_SC_LEFT_ARROW;

	if (buttons & BUTTON_R)
		KeyboardReport->KeyCode[UsedKeyCodes++] = HID_KEYBOARD_SC_RIGHT_ARROW;
		
	if (buttons & BUTTON_TL)
		KeyboardReport->KeyCode[UsedKeyCodes++] = HID_KEYBOARD_SC_D;
		
	if (buttons & BUTTON_TR)
		KeyboardReport->KeyCode[UsedKeyCodes++] = HID_KEYBOARD_SC_C;

	*ReportSize = sizeof(USB_KeyboardReport_Data_t);
	return false;
}

/** HID class driver callback function for the processing of HID reports from the host.
 *
 *  \param[in] HIDInterfaceInfo  Pointer to the HID class interface configuration structure being referenced
 *  \param[in] ReportID    Report ID of the received report from the host
 *  \param[in] ReportType  The type of report that the host has sent, either HID_REPORT_ITEM_Out or HID_REPORT_ITEM_Feature
 *  \param[in] ReportData  Pointer to a buffer where the created report has been stored
 *  \param[in] ReportSize  Size in bytes of the received HID report
 */
void CALLBACK_HID_Device_ProcessHIDReport(USB_ClassInfo_HID_Device_t* const HIDInterfaceInfo,
                                          const uint8_t ReportID,
                                          const uint8_t ReportType,
                                          const void* ReportData,
                                          const uint16_t ReportSize)
{
/*	uint8_t  LEDMask   = LEDS_NO_LEDS;
	uint8_t* LEDReport = (uint8_t*)ReportData;

	if (*LEDReport & HID_KEYBOARD_LED_NUMLOCK)
	  LEDMask |= LEDS_LED1;

	if (*LEDReport & HID_KEYBOARD_LED_CAPSLOCK)
	  LEDMask |= LEDS_LED3;

	if (*LEDReport & HID_KEYBOARD_LED_SCROLLLOCK)
	  LEDMask |= LEDS_LED4;

	LEDs_SetAllLEDs(LEDMask);*/

	// do nothing with this, for now
}
